#!/usr/bin/perl

#####
# This perl script will make a copy of local files, compress it
# and store it to the remote ftp. 
# It will group them to '<YYMM>' folders. 
#
# Algorithm:
# ====================
# 1 - Read the config file
# 2 - Check the last archive time from the the status file
# 3 - Find files between last archive and current time
# 4 - Create sub folder for the file group (if needed)
# 5 - Navigate to the archive dest folder
# 6 - Transfer to the dest folder
#
# Author : Abd Hafiizh Wahab
# Last modified : Mon Aug  9 11:03:08 MYT 2010
# Changed the IP to temp shared folder IP
#
##


use strict;
use warnings;
use FindBin qw ($RealBin);
use Net::FTP;
use File::Basename;
use File::stat;
use Time::localtime;

my $configFile = "$RealBin/csg2log.conf"; 
my $logTimeFile = "$RealBin/csg2log.stat";
#my $ftpURL = '172.18.29.170';
my $ftpURL = '172.16.48.42';
my $ftpUser = 'vendor2';
my $ftpPasswd = 'vendor2@1234';


my $DEBUG = 1;

my $hostname = qx(hostname);
chomp($hostname);

open FH, $configFile or die "Could not open $configFile";

my @hostAllConfigs;
while (<FH>){
	s/^\s*//;
	push( @hostAllConfigs, $_ ) if m/^$hostname/;
}
close FH or die $!;


# log file is stored in 'param=value'
my (@logLine,%logHash);
if ( -e $logTimeFile ) {
	open FH, "<$logTimeFile" or die $!;
	while (<FH>){
		chomp;
		@logLine = split(/=/, $_);
		$logHash{$logLine[0]} = $logLine[1];
	}

} else {
	open FH, ">$logTimeFile" or die $!;
}
close FH or die $!;


# Connecting to FTP
my $ftp = Net::FTP->new($ftpURL) 
	or die "Cannot connect to $ftpURL : $@";
$ftp->login("$ftpUser","$ftpPasswd") 
	or die "Cannot login : ", $ftp->message;

$ftp->binary();


# Process the config line by line 
# Field 0 : hostname
# Field 1 : log file fullname 
# Field 2 : archive dir

my $configNo = 0;
foreach (@hostAllConfigs){

	if ($DEBUG) {
		print "=============================\n".
		"Time 	   : " . ctime(time) . "\n".
		"Config No : " . ++$configNo . "\n";
	}
	
	my @hostConfig;
	my ($logsArDir,$logFullName); 

	@hostConfig = split(/\s+/ , $_);
	$logsArDir = $hostConfig[2];
	$logFullName = $hostConfig[1];


	# check for the last backup time
	# if it's the first time, create a very old date 
	# (so that all files will be backup)
	my $lastArchiveTime;

	if (exists $logHash{$logFullName}) {
		$lastArchiveTime = $logHash{$logFullName};
	} else {
		$lastArchiveTime = '190301020304';
	}

	
	my $status;
	$status = system("touch -t $lastArchiveTime $logTimeFile");
	die "Failed to create file." if ($status >>=8 != 0);

	my @gzipList;
	my $logPattern = basename($logFullName);
	my $logPath = dirname($logFullName);

	@gzipList = qx(find $logPath/* -prune -name '$logPattern' -type f -newer $logTimeFile);
	if ($DEBUG) {print "Gzip list : \n @gzipList \n"};


	my $currTime;
	$currTime = sprintf("%.4d%.2d%.2d%.2d%.2d", 
		localtime->year + 1900,
		localtime->mon + 1,
		localtime->mday,
		localtime->hour,
		localtime->min
	);
	
	$logHash{$logFullName} = $currTime;


	my $archiveName = $logPattern;
	$archiveName =~ s/\*//g;
	
	if ($DEBUG) {print "Archive basename : $archiveName \n"};
	# Gzip and move files one by one one to save resources.
	my $curr_dir = '';
	foreach (@gzipList){

		chomp;
		if ($DEBUG) {print "Current list is $_ \n"};

		my $modTimeObj = localtime(stat($_)->mtime);
		my $fileModTime = sprintf("%.4d-%.2d-%.2d_%.2dh%.2dm%.2ds", 
			$modTimeObj->year + 1900,
			$modTimeObj->mon + 1,
			$modTimeObj->mday,
			$modTimeObj->hour,
			$modTimeObj->min,
			$modTimeObj->sec
		);

		my $currArchiveName = sprintf("%s_%s.gz",
			$archiveName,
			$fileModTime
		);

		# Dont gzip if it's already gzipped
		if (/\.gz$/) {
			$status = system ("cp $_ /tmp/$currArchiveName");
		} else {
			$status = system("cat $_ | gzip -9 > /tmp/$currArchiveName");
		}
		die "Cannot create the file." if ($status >>= 8 != 0);

		my $arDir;
		$arDir = sprintf("%s/%d%.2d",
			$logsArDir,
			$modTimeObj->year + 1900,
			$modTimeObj->mon + 1,
		);

		if ($DEBUG) {print "Archive Dir is $arDir \n"};

		# If it's not the current dir ,navigate and create if needed
		if ($curr_dir ne $arDir){
			
			$ftp->cwd('/') or die "cannot change to the root dir";
			foreach my $parentDir ( split('/',$arDir) ) {
				next if $parentDir eq '';

				$curr_dir = $ftp->pwd();
				$ftp->cwd($parentDir);

				if ($ftp->pwd() eq $curr_dir){
					$ftp->mkdir($parentDir) 
						or die "Cannot mkdir '$parentDir' :",$ftp->message; 
					$ftp->cwd($parentDir);
				}
			}
		}

			
		if ($DEBUG) {printf "Current Dir is %s \n", $ftp->pwd() };
		if ($DEBUG) {print "Putting the file ... \n"};

		$ftp->put("/tmp/$currArchiveName");
		
		if ($DEBUG) {print "Removing the temp file : /tmp/$currArchiveName ... \n"};
		unlink("/tmp/$currArchiveName");
	}
}


open FH, ">$logTimeFile" or die $!;

# Update the status file
foreach (keys %logHash){
	print FH "$_=$logHash{$_}\n";
}

# Disconnect the session
$ftp->quit();
close FH or die $!;


